#!perl -w
#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2008-2009 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################

##########################################################################
#                                                                        # #
# Rational Test RealTime                                                 #   #
#                                                                        #######
#            -*-  Source Instrumentation launcher for Rhapsody -*-             #
#                                                                              #
#                                                                              #
################################################################################

#$Id: RhapTestRT.pl,v 1.3.4.1 2009/09/04 09:02:36 rfrance Exp $

# generate the local TestRT.bat to set compiler/linker and ATLTGT
# then generate the option file
# then check that %1.bat has enough %x on the make command line
#RhapTestRT.exe -bat "%2.opt" "%RHAPSODYSHAREETC%\Env2TDP.txt" "%RHAPSODYSHAREETC%\%1.bat"

die "Environment variable TESTRTDIR not found. Exiting.\n\n" if(!defined($ENV{'TESTRTDIR'})); # NO TRANSLATION
$TESTRTDIR="$ENV{'TESTRTDIR'}";
#$SCRIPTDIR="$TESTRTDIR/lib/scripts/";

################################################################################
# useful for stdout/stderr direct viewing inside 'console' of studio window
# (better: we have stdout/stderr lines in right order and not stder before stdout
use FileHandle;
use File::Copy ;

STDOUT->autoflush(1);
STDERR->autoflush(1);

$RHAP_NAME="root";
#$RHAP_FILE='HomeAlarm.rpy';
#$RHAP_DIR='.';
$RHAP_CONF_PATH=".";
$RHAP_LANG='CPP';
#$RHAP_MAKEFILE='';
$RHAP_ENVIRONMENT="";
$RHAP_COV='-BLOCK=Logical -Proc=Ret';
$RHAP_MEMPRO='False';
$RHAP_PERFPRO='False';
$RHAP_TRACE='False';
$RHAP_LIBS='$OMROOT';
$RHAP_USEROPTIONS='';
$RHAP_USERSETTINGS='';
$RHAP_TARGET='<default>';
%RHAP_TAGS=();
require "RhapConf.pl";

my $batType=$ARGV[0];
my $optionFile=$ARGV[1]; $optionFile=~s/\"//g;
my $eqFile=$ARGV[2];     $eqFile=~s/\"//g;
my $makBatFile=$ARGV[3]; $makBatFile=~s/\"//g;

#print "+$RHAP_ENVIRONMENT\n";
#print "+$RHAP_CONF_PATH\n";
#print "+$RHAP_NAME\n";
#print "+$optionFile\n";
#print "+$eqFile\n";
#print "+$makBatFile\n";

#create directory for TestRT static files
my $TestRTstaticDir=$RHAP_CONF_PATH.'/TestRT';
unless(-d $TestRTstaticDir)  {
	mkdir ($TestRTstaticDir,0777);
}

# load defaults TDP list according to Rhapsody property CPP_CG.Configuration.environment
my $CC="";#="\$(CC)";
my $CPP="";#="\$(CPP)";
my $LINK_CMD="";#="\$(LINK_CMD)";

if(open (TDP, "$eqFile")){
  while(<TDP>){
    if(/^\s*\Q$RHAP_ENVIRONMENT\E\s+(.*)$/){
    	my $values=$1;
    	if($values=~/ATLTGT=\"(.+)\"/){
		my $tgt=$1;
		$tgt=$TESTRTDIR.'/targets/'.$tgt unless($tgt=~m/[\\\/]/);
		$ENV{'ATLTGT'}=$tgt;
    	}
    	elsif($values=~/ATLTGT=(\w+)\s/){
		my $tgt=$1;
		$tgt=$TESTRTDIR.'/targets/'.$tgt;
		$ENV{'ATLTGT'}=$tgt;
    	}
    	if($values=~/CC=\"(.+)\"/){
		$CC=$1;
    	}
    	elsif($values=~/CC=([^\s]+)(\s)?/){
		$CC=$1;
    	}
    	if($values=~/CPP=\"(.+)\"/){
		$CPP=$1;
    	}
    	elsif($values=~/CPP=([^\s]+)(\s)?/){
		$CPP=$1;
    	}
    	if($values=~/LINK_CMD=\"(.+)\"/){
		$LINK_CMD=$1;
    	}
    	elsif($values=~/LINK_CMD=([^\s]+)(\s)?/){
		$LINK_CMD=$1;
    	}
    	last;
    }
  }
  close TDP;
} else { print "unable to read $eqFile\n";}

#write option file regarding RTRT_xx tags and properties
if(open (OPT,">$optionFile")) {
  print OPT "--NAME=$RHAP_NAME\n";
  print OPT "-FDCDIR=\"$TestRTstaticDir\"\n-TSFDIR=\"$TestRTstaticDir\"\n";

  foreach my $tag (keys %RHAP_TAGS){
	if($tag=~/^RTRT_/){
		if($tag eq "RTRT_Target"){
			print "Targtet forced by tag RTRT_Target\n";
			my $tgt=$RHAP_TAGS{$tag};
			$tgt=$TESTRTDIR.'/targets/'.$tgt unless($tgt=~m/[\\\/]/);
			if(-d $tgt) {
			    $ENV{'ATLTGT'}=$tgt;
			    $CC=""; $CPP=""; $LINK_CMD="";
			} else {
			    print "Unnown TDP ",$tgt,"\n";
			}
		}
		elsif($tag eq "RTRT_CC"){
			$CC=$RHAP_TAGS{$tag};
		}
		elsif($tag eq "RTRT_CPP"){
			$CPP=$RHAP_TAGS{$tag};
		}
		elsif($tag eq "RTRT_LINK_CMD"){
			$LINK_CMD=$RHAP_TAGS{$tag};
		}
		else
		{
			print OPT $RHAP_TAGS{$tag},"\n";
		}
	}
  }
  if(($RHAP_TARGET ne "<default>") && ($RHAP_TARGET ne "")){
	print "Target forced by Target property\n";
	my $tgt=$RHAP_TARGET;
	$tgt=$TESTRTDIR.'/targets/'.$tgt unless($tgt=~m/[\\\/]/);
	if(-d $tgt) {
	    $ENV{'ATLTGT'}=$tgt;
	    $CC=""; $CPP=""; $LINK_CMD="";
	} else {
	    print "Unnown TDP ",$tgt,"\n";
	}
  }

  if($batType=~m/f$/i){ #batF or shF for the frameWork build
	print OPT "-memprolib " if($RHAP_MEMPRO eq 'True');
  } else {
	print OPT "-mempro " if($RHAP_MEMPRO eq 'True');
	print OPT "-perfpro " if($RHAP_PERFPRO eq 'True');
	print OPT "-trace " if($RHAP_TRACE eq 'True');
	print OPT $RHAP_COV,"\n";
	print OPT "-noinstrdir=",$RHAP_LIBS,"\n" if($RHAP_LIBS ne '');
  }
  print OPT "-ignore=TestRealTime.cpp,TestRealTime.c\n";
  print OPT $RHAP_USEROPTIONS, "\n" if($RHAP_USEROPTIONS  ne '');
  print OPT $RHAP_USERSETTINGS,"\n" if($RHAP_USERSETTINGS ne '');
  close OPT;
} else {print "unable to write $optionFile\n";}

#print "-- $CC --$CPP --$LINK_CMD\n";
unless($CC){
   print "getting compiler and linker from TDP\n"; #from tp.ini and tpcpp.ini
   if(defined($ENV{'ATLTGT'}) && (-d $ENV{'ATLTGT'})){
	if(open INI,"$ENV{'ATLTGT'}/tpcpp.ini"){
	 while(<INI>){
		if(/^COMPILER=\"(.*)\"/) {$CPP=$1 unless($CPP);}
		if(/^LINKER=\"(.*)\"/) {$LINK_CMD=$1 unless($LINK_CMD);}
	 }
	close INI;
	}
	if(open INI,"$ENV{'ATLTGT'}/tp.ini"){
	 while(<INI>){
		if(/^COMPILER=\"(.*)\"/) {$CC=$1;}
		if(/^LINKER=\"(.*)\"/) {$LINK_CMD=$1 unless($LINK_CMD);}
	 }
	close INI;
	}
   }
}
#print "-- $CC --$CPP --$LINK_CMD\n";
$CPP=$CC unless($CPP);
$LINK_CMD=$CC unless($LINK_CMD);

#write the TestRT.bat
my $batext="bat";
my $set="set";
my $prefix='%';
if($batType=~m/^-sh/){
$batext="sh";
$set="";
$prefix='$';
}

if(($CC ne "") && open BAT, ">TestRT.$batext") {
	$CC=$CPP if($RHAP_LANG eq 'CPP');
	print BAT "$set CC=$CC\n";
	print BAT "$set CPP=$CPP\n";
	print BAT "$set LINK_CMD=$LINK_CMD\n";
	print BAT "$set ATLTGT=$ENV{'ATLTGT'}\n";
	print BAT "$set RTRTMEMPRO=$RHAP_MEMPRO\n";
	close BAT;
} else {print "unable to write TestRT.$batext\n";}

#check the build batch file
if(open(MAK,$makBatFile)){
    while(<MAK>){
	if(/make/ && /\Q$prefix\E1/){
	    unless(/\Q$prefix\E9/){
		print "WARNNING: the file $makBatFile may not be completed\n";
		print "make sur ${prefix}3 to ${prefix}9 is on the line:\n$_\n";
	    }
	    last;
	}
    }
    close MAK;
}

exit(0);
1;