#!perl -w

#################################################################################
#
#  Licensed Materials - Property of IBM
#  Restricted Material of IBM
#  IBM Rational Test RealTime
#  (c) Copyright IBM Corp. 2003-2009 All Rights Reserved.
#
#  US Governement Users Restricted Rights - Use, duplication or
#  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#################################################################################


#################################################################################
# This file expects the name of a parameter file its first and only argument
##
# It will return
# 0 The test executed correctly
# 1 The user interface could not be run
# 5 The project could not be built
#################################################################################

use warnings;
use strict;

# Set to one to enable debugging
my $DEBUG = 1;

# The script expects one parameter which is the name of the parameter file
die "$0 needs one, and only one parameter which is the name of the file containing the parameters to run the test\n" unless (@ARGV == 1);
my $paramFile = $ARGV[0];
my $Drive;
my $errMount="";
# Does the parameter file exist andd is it readable?
open (IN, $paramFile) || die "Cannot open the file $paramFile for reading:\n$!\n";

my %params;
my $lineno = 0;
while (<IN>) {
  $lineno++;
  next if (/^\s*$/); # Ignore empty lines
  next if (/^\s*#/); # Ignore comments
  chomp; # Remove newline characters
  # now must have a line with the format key=value
  if (m,\s*(\w+)=(.+)\s*$,) {
    my $key = $1;
    my $value = $2;
    # Store the key and value in the parameter lookup table
    $params{$key} = $value;
  } else {
    die "Unrecognized line in the parameter file\n$lineno: $_\n";
  }
}
close (IN);

# Check that we have received the minimum expected keys
#die "Missing the name of the test node to execute. Expected parameter name is \"testNodeName\".\n" unless (exists $params{testNodeName});
die "Missing the name of the project file containing the tests. Expected parameter name is \"testRTScriptPath\".\n" unless (exists $params{testRTScriptPath});

# Remove double backslashes from the name of the project
if($params{sharedPrefix}) {
	$params{sharedPrefix} =~ s/\\\\/\\/g;
	$params{sharedPrefix} =~ s/\\:/:/g;
	if(($params{sharedPrefix} =~ m/^\\\\/) && ($^O =~ /Win32/)) {
		my $mount="net use * ".$params{sharedPrefix};
		$DEBUG && print "Running the following command:\n$mount\n";
		my $out = `$mount` ;
		my $status = $? ;
		my $ret = $status / 256;
		if(($out=~ m/\s(\w\:)\s/) && ($ret ==0)){
			$Drive=$1;
		} else {
			$errMount = $out." error code ".$ret; 		
		}
	}
}

$params{testRTScriptPath} =~ s/\\\\/\\/g;
$params{testRTScriptPath} =~ s/\\:/:/g;
$params{testRTScriptPath} =~ s/^\Q$params{sharedPrefix}\E/$Drive/ if($Drive);

if($params{tempDirName}) {
	$params{tempDirName} =~ s/\\\\/\\/g;
	$params{tempDirName} =~ s/\\:/:/g;
	$params{tempDirName} =~ s/^\Q$params{sharedPrefix}\E/$Drive/ if($Drive);
}	

# Calculate the log file name
my $logfile = $params{testRTScriptPath};
$logfile =~s/([\\\/])[^\\\/]*$/$1TestRTRqm.log/i;
$DEBUG && print "logfile = $logfile\n";
open (LOG, ">$logfile") || die "Cannot open the log file $logfile\n";
print LOG $errMount;
# Now do what you want!

# Then run the test via studio
my $command;
if($params{testNodeName}) {
	$command = "studio -r \"".$params{testNodeName}."\" \"".$params{testRTScriptPath}."\"";
} else {
	$command = "studio -r  \"".$params{testRTScriptPath}."\"";
}
$command .= " -config \"".$params{tdpName}."\"" if (exists $params{tdpName});
$command .=  " -html \"".$params{tempDirName}."/htmlDir\"" if (-d $params{tempDirName});

$DEBUG && print "Running the following command:\n$command\n";
$! = 0;
my $status = system ($command);
my $ret = $status / 256;
$DEBUG && print "Raw return status $status => $ret\n";
# Check the return value and return one of the following values from the script
# 0 The test executed correctly
# 1 The user interface could not be run
# 5 The project could not be built
if ($ret == 1) {
  # There is an ambiguity with the value 1. 1 means an error occurred in system
  # or the TestRT test was built successfully. $! will tell us which it is
  if ($! eq "") {
    # TestRT test was built successfully
    $ret = 0;
  } else {
    # an error occured in system call
    print LOG "TestRTRqm.pl: When trying to run the Test RealTime user interface the following error code was returned: $ret\n$!\n\n";
    $ret =1;
  }
} elsif ($ret == 5) {
  # Build failed
  print LOG "TestRTRqm.pl: Build of the test failed with error code 5\n";
} else {
  print LOG "TestRTRqm.pl: When trying to run the Test RealTime user interface the following error code was returned: $ret\n$\n!\n" unless ($ret == 0);
  $ret = 1;
}

close (LOG);
system("net use $Drive /delete") if($Drive);
$DEBUG && print "exit code = $ret\n";
exit($ret);

