

/*
 *  Licensed Materials - Property of IBM
 *  Restricted Material of IBM
 *  IBM Rational Test RealTime
 *  (c) Copyright IBM Corp. 2003-2007 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 */


/*******************************************************************************
* $Id: threads.c,v 1.32 2008/05/05 12:43:24 rfrance Exp $
* $Descr: Threads Management (included by TP.c[pp]) $
*******************************************************************************/


#if ATL_MULTI_THREADS

/* proto declared in the rtstack.h included later in TP.c[pp] */
_ATL_EXTERN_C ATL_TP_PROTO_PREFIX     RTRT_THIDX_T _atl_threads_get_index       ( );

enum {
        _ATL_THREADS_CREATED,
        _ATL_THREADS_CONTINUE,
        _ATL_THREADS_SCHEDULED
};

#define THREADINDEX() RTRT_THIDX_T thread_index;

/*******************************************************************************
* Global variables
*******************************************************************************/

RTRT_GLOBALVAR RTRT_MT_THREAD_SELF_T _atl_threads_table_used[ATL_THREADS_MAX+1];

RTRT_GLOBALVAR RTRT_MT_THREAD_SELF_T _atl_threads_current;
RTRT_GLOBALVAR RTRT_THIDX_T _atl_threads_index;
RTRT_GLOBALVAR RTRT_THIDX_T atl_threads_index_max; /* can be used by the custom specific data */

RTRT_MT_CRITICAL_VAR_DECL(atl_index_critical)

/* DATA_DECLARE must be just before function body to let custom declare a function body as well */
RTRT_GLOBALVAR RTRT_MT_THREAD_SPECIFIC_DATA_DECLARE

RTRT_STATICFCT void _atl_stack_finish ( RTRT_THIDX_T );

/*******************************************************************************
*  Service: _atl_threads_release_idx
*  Goal: called automatically by the OS when a thread terminates
*         (if implemented)
*******************************************************************************/
_ATL_C_GLOBALFCT void _atl_threads_release_idx
#if !RTRT_KR
( RTRT_THIDX_T idx )
#else
( idx )
  RTRT_THIDX_T idx;
#endif
{
 RTRT_MT_THREAD_SELF_T self;
#if USE_ATP || USE_ATQ || USE_ATT
  if ( idx != 0 )
  {
    _atl_stack_finish (idx);
  }
#endif /* USE_ATP || USE_ATQ || USE_ATT */
  self=_atl_threads_table_used[idx]; /* may be used by the custom specific data */
  RTRT_MT_THREAD_SPECIFIC_DATA_SET ( 0 );
  _atl_threads_table_used[idx] = 0;
}

/*******************************************************************************
*  Service: _atl_threads_look_for_index
*******************************************************************************/
_ATL_C_GLOBALFCT RTRT_THIDX_T atl_threads_look_for_index
#if !RTRT_KR
( RTRT_MT_THREAD_SELF_T self )
#else
( self )
RTRT_MT_THREAD_SELF_T self;
#endif
{
	/* find the index of this thread */
    RTRT_THIDX_T thedata;
	for(thedata=0;
            thedata <= atl_threads_index_max && _atl_threads_table_used[thedata] != self;
            thedata++ );
        if ( thedata > atl_threads_index_max )
        {
           thedata=0;
        }
  return thedata;
}

/*******************************************************************************
*  Service: _atl_threads_release_index
*  Goal: called automatically by the OS when a thread terminates
*         (if implemented)
*******************************************************************************/
_ATL_C_GLOBALFCT void _atl_threads_release_index
#if !RTRT_KR
( void *thedata )
#else
( thedata )
  void *thedata;
#endif
{
 _atl_threads_release_idx((RTRT_THIDX_T)RTRT_PTR_TO_INT(thedata));
}

/*******************************************************************************
*  Service: _atl_threads_kill
*  Goal: called with self as parameter when a thread is killed
*         (if implemented)
*******************************************************************************/
#ifdef NEEDS_threads_kill
_ATL_C_GLOBALFCT void _atl_threads_kill
#if !RTRT_KR
( RTRT_MT_THREAD_SELF_T self )
#else
( self )
 RTRT_MT_THREAD_SELF_T self;
#endif
{
    RTRT_THIDX_T thedata;
	/* find the index of this thread */
    thedata=atl_threads_look_for_index(self);
        if ( thedata )
        {
            _atl_stack_finish (thedata );
	    	_atl_threads_table_used[thedata] = 0;
        }
        else
        {
           ATL_ERROR ('u', "try to kill unknown thread" );
        }
}
#endif /* NEEDS_threads_kill */

/*******************************************************************************
*  Service: _atl_threads_get_index
*******************************************************************************/
_ATL_C_GLOBALFCT RTRT_THIDX_T _atl_threads_get_index
#if !RTRT_KR
( void )
#else
( )
#endif
{
  RTRT_MT_THREAD_SELF_T self,cur;
  RTRT_THIDX_T thedata;

  self = RTRT_MT_THREAD_SELF;
  RTRT_MT_CRITICAL_VAR_LOCK(atl_index_critical)
  thedata= _atl_threads_index;
  cur =_atl_threads_current;
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_index_critical)
  if ( self != cur )
  {
	  RTRT_MT_THREAD_SPECIFIC_DATA_GET ( thedata );

	  if ( _atl_threads_table_used[thedata] != self )
	  {
		ATL_ERROR('q',"_atl_threads_get_index Unknow thread");
		thedata=0;
	  }
  }
  return thedata;
}
/*******************************************************************************
*  Service: _atl_threads_get_status
*    returns _ATL_THREADS_CREATED if thread is new,
*            _ATL_THREADS_SCHEDULED if it has just been scheduled,
*            _ATL_THREADS_CONTINUE else
*******************************************************************************/
_ATL_C_GLOBALFCT int _atl_threads_get_status
#if !RTRT_KR
( RTRT_THIDX_T *cur_idx )
#else
( cur_idx)
RTRT_THIDX_T *cur_idx;
#endif
{
  RTRT_MT_THREAD_SELF_T self;
  RTRT_THIDX_T i;
  RTRT_THIDX_T thedata;

  if(*cur_idx)
  {
	  if(*cur_idx==_atl_threads_index)
	  {
		  return _ATL_THREADS_CONTINUE;
	  }
	  thedata=*cur_idx;
  }

  self = RTRT_MT_THREAD_SELF;
  RTRT_MT_CRITICAL_VAR_LOCK(atl_index_critical)

  if ( self == (RTRT_MT_THREAD_SELF_T)0 )
  { /* This could be the main process: first item of the table */
    _atl_threads_current = 0;
    _atl_threads_index = 0;
    *cur_idx=0;
    RTRT_MT_CRITICAL_VAR_UNLOCK(atl_index_critical)
    return _ATL_THREADS_CONTINUE;
  }
  if ( self == _atl_threads_current )
  { /* No scheduling has been done between threads */
    *cur_idx=_atl_threads_index;
    RTRT_MT_CRITICAL_VAR_UNLOCK(atl_index_critical)
    return _ATL_THREADS_CONTINUE;
  }

  if(*cur_idx==0)
  {
  	RTRT_MT_THREAD_SPECIFIC_DATA_GET ( thedata );
  }
  _atl_threads_index = thedata;
  _atl_threads_current = self;

  if ( _atl_threads_table_used[thedata] == self )
  { /* This thread is woken up */
    *cur_idx=thedata;
    RTRT_MT_CRITICAL_VAR_UNLOCK(atl_index_critical)
    return _ATL_THREADS_SCHEDULED;
  }

  /* This thread is just created: let's give him a data space */
  for ( i = 1;
        i <= atl_threads_index_max && _atl_threads_table_used[i];
        i++ );
  if ( i > ATL_THREADS_MAX )
  { /* Error : no space enough in table ...
     * let's use last one, just can't stop */
    ATL_ERROR ('h', "Maximum number of threads reached" );
    i = ATL_THREADS_MAX;
  }
  _atl_threads_index = i;
  if(atl_threads_index_max < i) atl_threads_index_max =i;
  _atl_threads_table_used[i] = self;
  RTRT_MT_CRITICAL_VAR_UNLOCK(atl_index_critical)
  thedata = i;
  RTRT_MT_THREAD_SPECIFIC_DATA_SET ( thedata )
  if(*cur_idx)
  {
	  ATL_ERROR('F',"_atl_threads_get_status CREATION WARNING");
  }
  *cur_idx=thedata;
  return _ATL_THREADS_CREATED;
}

/*******************************************************************************
*  Service: _atl_threads_dying
*  Goal: called automatically without parameter by the OS when a thread terminates
*         (if implemented)
*******************************************************************************/
_ATL_C_GLOBALFCT void _atl_threads_dying
#if !RTRT_KR
( void )
#else
( )
#endif
{
	_atl_threads_release_idx(_atl_threads_get_index());
}

/*******************************************************************************
*  Service: _atl_threads_init
*******************************************************************************/
_ATL_C_GLOBALFCT void _atl_threads_init
#if !RTRT_KR
( void )
#else
( )
#endif
{
#if ! RTRT_RAM_CLEAN

  RTRT_THIDX_T i;

  /* global variables initialization */
  _atl_threads_current = 0;
  _atl_threads_index = 0;
  /* _atl_threads_key creation */

  /* global usage table initialization */
  for ( i = 0;
        i <= ATL_THREADS_MAX;
        i++ )
  {
    _atl_threads_table_used[i] = 0;
  }
  atl_threads_index_max=0;
#endif
  RTRT_MT_CRITICAL_VAR_INIT(atl_index_critical) /* mutex protecting the thread_index manipulations */

  RTRT_MT_THREAD_SPECIFIC_DATA_CREATE
}

/*******************************************************************************
*  Service: _atl_threads_init
*******************************************************************************/
_ATL_C_GLOBALFCT void _atl_threads_late_exit
#if !RTRT_KR
( void )
#else
( )
#endif
{
  RTRT_MT_THREAD_SPECIFIC_DATA_DELETE
}
#else /* ATL_MULTI_THREADS */

#define THREADINDEX()

#endif /* ATL_MULTI_THREADS */

